package net.pricefx.common.apibuilder.rebateagreement;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import net.pricefx.common.api.InputType;
import net.pricefx.common.apibuilder.AbstractBuilder;
import net.pricefx.common.apibuilder.IdGenerator;
import net.pricefx.common.apibuilder.Invocations;
import net.pricefx.common.apibuilder.MethodInvocation;
import net.pricefx.common.dto.rat.RATPlaceholder;

/**
 * Created by petercipov on 22/03/16.
 */
public class RATBuilder extends AbstractBuilder {

    public static final String NAME = "RATBuilder";
    public static final String METHOD_FROM_EXISTING = "fromExisting";
    public static final String METHOD_FROM_PARAMS = "fromParams";
    public static final String METHOD_BUILD = "build";
    
    private final List<RATPlaceholder> placeholders;
    private final String templateTypeId;
    private final String targetRAUniqueName;

    public RATBuilder(IdGenerator idGenerator, Invocations<MethodInvocation> invocations, String templateTypeId, String targetRAUniqueName) {
        super(idGenerator.getNext(), NAME, idGenerator, invocations);
        this.templateTypeId = templateTypeId;
        this.targetRAUniqueName = targetRAUniqueName;
        this.placeholders = new LinkedList<>();
    }

    public String templateTypeId() {
        return templateTypeId;
    }

    public String targetRAUniqueName() {
        return targetRAUniqueName;
    }

    public RABuilder fromExisting(String RAuniqueName) {
        RABuilder builder = new RABuilder(this);
        addInvocation(METHOD_FROM_EXISTING, this, builder, RAuniqueName);

        return builder;
    }

    @SuppressWarnings("rawtypes")
	public RABuilder fromParams(Map params) {
        RABuilder builder = new RABuilder(this);
        addInvocation(METHOD_FROM_PARAMS, this, builder, params);

        return builder;
    }

    @Override
    public RebateAgreementBuilderResult build() {
    	addInvocation(METHOD_BUILD, this);
        return new RebateAgreementBuilderResult();
    }

    public RATBuilder registerPlaceholder(InputType type, String name) {
        this.placeholders.add(new RATPlaceholder(name, type));
        return this;
    }

    public List<RATPlaceholder> placeholders() {
        return placeholders;
    }

    public static class RABuilder extends BasicRebateAgreementBuilder {

        public static final String NAME = "RABuilder";
        public static final String METHOD_ADD_LINE_ITEM_WITH_ID = "addLineItemWithId";
        public static final String METHOD_APPLY_PLACEHOLDER = "applyPlaceholder";
        public static final String METHOD_SUBMIT_AFTER_BUILD = "submitAfterBuild";


        public RABuilder(RATBuilder ratBuilder) {
            super(ratBuilder.idGenerator.getNext(), NAME, ratBuilder.idGenerator, ratBuilder.invocations, null);
        }

        public String getIstanceId() {
            return this.instanceId;
        }

        public RABuilder addLineItemWithId(String lineId, String rebateType) {
        	addInvocation(METHOD_ADD_LINE_ITEM_WITH_ID, ROOT_LINE_ID, lineId, rebateType);
            return this;
        }

        public RABuilder addLineItemWithId(String parentId, String lineId, String rebateType) {
        	addInvocation(METHOD_ADD_LINE_ITEM_WITH_ID, parentId, lineId, rebateType);
            return this;
        }

        public RABuilder applyPlaceholder(String placeholder, Object value) {
        	addInvocation(METHOD_APPLY_PLACEHOLDER, placeholder, value);
            return this;
        }

        public RABuilder submitAfterBuild() {
        	addInvocation(METHOD_SUBMIT_AFTER_BUILD);
            return this;
        }

        @Override
        public String toString() {
            return "Instance="+instanceId;
        }
    }

}
