package net.pricefx.common.dto.creationworkflow;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

public class CreationWorkflowStepDefinition {
	private String label;
	private List<String> userAssigneeLoginNames = Collections.emptyList();
	private List<String> userGroupAssigneeUniqueNames = Collections.emptyList();
	private List<String> userWatcherLoginNames = Collections.emptyList();
	private List<String> userGroupWatcherUniqueNames = Collections.emptyList();
	private boolean shouldSendNotification = true;

	public String getLabel() {
		return label;
	}

	public CreationWorkflowStepDefinition withLabel(String label) {
		this.label = label;
		return this;
	}

	@Deprecated
	public String getUserAssignee() {
		return Optional.ofNullable(userAssigneeLoginNames).map(s -> !s.isEmpty() ? s.get(0) : null).orElse(null);
	}

	public List<String> getUserAssignees() {
		return userAssigneeLoginNames;
	}


	public CreationWorkflowStepDefinition withUserAssignee(String userAssigneeLoginName) {
		this.userAssigneeLoginNames = Collections.singletonList(userAssigneeLoginName);
		return this;
	}

	public CreationWorkflowStepDefinition withUserAssignees(String... userAssigneeLoginName) {
		this.userAssigneeLoginNames = Arrays.asList(userAssigneeLoginName);
		return this;
	}

	public CreationWorkflowStepDefinition withUserWatchers(String... userWatcherLoginNames) {
		this.userWatcherLoginNames = Arrays.asList(userWatcherLoginNames);
		return this;
	}

	public CreationWorkflowStepDefinition withUserGroupWatchers(String... userGroupWatcherUniqueNames) {
		this.userGroupWatcherUniqueNames = Arrays.asList(userGroupWatcherUniqueNames);
		return this;
	}

	public List<String> getUserWatchers() {
		return userWatcherLoginNames;
	}

	public List<String> getUserGroupWatchers() {
		return userGroupWatcherUniqueNames;
	}

	@Deprecated
	public String getUserGroupAssignee() {
		return Optional.ofNullable(userGroupAssigneeUniqueNames).map(s -> !s.isEmpty() ? s.get(0) : null).orElse(null);
	}

	public List<String> getUserGroupAssignees() {
		return userGroupAssigneeUniqueNames;
	}

	public CreationWorkflowStepDefinition withUserGroupAssignee(String userGroupAssigneeUniqueName) {
		this.userGroupAssigneeUniqueNames = Collections.singletonList(userGroupAssigneeUniqueName);
		return this;
	}

	public CreationWorkflowStepDefinition withUserGroupAssignees(String... userGroupAssigneeUniqueNames) {
		this.userGroupAssigneeUniqueNames = Arrays.asList(userGroupAssigneeUniqueNames);
		return this;
	}

	public boolean getShouldSendNotification() {
		return shouldSendNotification;
	}

	public CreationWorkflowStepDefinition withShouldSendNotification(boolean shouldSendNotification) {
		this.shouldSendNotification = shouldSendNotification;
		return this;
	}
}
