package net.pricefx.logic.core.model;

import com.fasterxml.jackson.databind.node.ObjectNode;
import net.pricefx.pckg.filesystem.ItemMarkers;
import net.pricefx.pckg.processing.ProcessingMarkers;
import net.pricefx.pckg.transform.TransformCalculationLogic;
import net.pricefx.pckg.transform.TransformWorkflowFormula;
import net.pricefx.pckg.transform.descriptor.TypeDescriptor;
import org.jetbrains.annotations.NotNull;

import java.nio.file.Path;
import java.util.List;
import java.util.Map;

public class ObjectHolder extends BaseMapNullable {

	protected Path dirPath;

	public ObjectHolder() {
		super();
	}

	public static <T extends ObjectHolder> T fromObjectNode(ObjectNode objectNode, Class<T> clazz, @NotNull Path modulePath) {
		T objectHolder = fromObject(objectNode, clazz);
		objectHolder.setDirPathBasedOnFsId(modulePath);
		return objectHolder;
	}

	public static ObjectHolder[] convertObjectHolderArray(TypeDescriptor typeDescriptor, List<ObjectNode> nodes, Path modulePath) {
		ObjectHolder[] objects;
		if (typeDescriptor == TransformCalculationLogic.DESCRIPTOR || typeDescriptor == TransformWorkflowFormula.DESCRIPTOR) {
			objects = new AbstractFormula[nodes.size()];
			for (int i = 0; i < nodes.size(); i++) {
				objects[i] = AbstractFormula.createLogic(nodes.get(i), typeDescriptor, modulePath);
			}
		} else {
			objects = new ObjectHolder[nodes.size()];
			for (int i = 0; i < nodes.size(); i++) {
				objects[i] = fromObjectNode(nodes.get(i), ObjectHolder.class, modulePath);
			}
		}
		return objects;
	}

	public void setDirPath(Path dirPath) {
		this.dirPath = dirPath;
	}

	public Path getDirPath() {
		return dirPath;
	}

	public String getDirPathBasedOnFsId() {
		Map procMap = ((Map) get(ProcessingMarkers.FIELD_PROCESS));
		if (procMap != null) {
			return (String) procMap.get(ItemMarkers.FIELD_fileSystemId);
		}
		return null;
	}

	public void setDirPathBasedOnFsId(Path modulePath) {
		String value = getDirPathBasedOnFsId();
		if (value != null) {
			setDirPath(modulePath.resolve(value instanceof String ? (String) value : null));
		}
	}
}
